if select(6, GetAddOnInfo("PitBull_" .. (debugstack():match("[i%.][t%.][B%.]ull\\Modules\\(.-)\\") or debugstack():match("[i%.][t%.][B%.]ull\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 61034 $"):match("%d+"))

local localization = (GetLocale() == "koKR") and {
	["Show up-to-date healing on the unit frames from all members."] = "유닛프레임에 모든 멤버의 최근의 치유량을 표시합니다.",
} or (GetLocale() == "zhCN") and {
	["Show up-to-date healing on the unit frames from all members."] = "在左右的友方目标窗体上显示虚拟治疗量.",
} or {}

local L = PitBull:L("PitBull-VisualHeal", localization)

local PitBull = PitBull
local PitBull_VisualHeal = PitBull:NewModule("VisualHeal", "LibRockHook-1.0")
PitBull_VisualHeal.desc = L["Show up-to-date healing on the unit frames from all members."]
PitBull_VisualHeal:RegisterPitBullModuleDependencies("HealthBar")
PitBull_VisualHeal:RegisterPitBullChildFrames('playerHealBar', 'incomingHealBar')
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2008-02-12 02:07:16 -0500 (Tue, 12 Feb 2008) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local PitBull_HealthBar

local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

local playerName = UnitName("player")
local LibHealComm

function PitBull_VisualHeal:OnEnable()
	LibHealComm = Rock("LibHealComm-3.0", false, true)
	if not LibHealComm then
		error("PitBull_VisualHeal requires LibHealComm-3.0 to be available.")
	end
	PitBull_HealthBar = PitBull:GetModule("HealthBar")
	
	self:AddHook(PitBull_HealthBar, "UpdateHealth")
	
	LibHealComm.RegisterCallback(self, "HealComm_DirectHealStart")
	LibHealComm.RegisterCallback(self, "HealComm_DirectHealUpdate")
	LibHealComm.RegisterCallback(self, "HealComm_DirectHealStop")
	LibHealComm.RegisterCallback(self, "HealComm_HealModifierUpdate")
end

function PitBull_VisualHeal:OnDisable()
	LibHealComm.UnregisterAllCallbacks(self)
end

local maxHealthEstimate = {
	WARRIOR = 4100,
	PALADIN = 4000,
	SHAMAN = 3500,
	ROGUE = 3100,
	HUNTER = 3100,
	DRUID = 3100,
	WARLOCK = 2300,
	MAGE = 2200,
	PRIEST = 2100,
}
local function EstimateUnitHealthMax(unit)
	local _, class = UnitClass(unit)
	local level = UnitLevel(unit) or 60
	
	return (maxHealthEstimate[class] or 4000) * level / 60
end

local playerIsCasting = false
local playerHealingTargetName = nil
local playerHealingSize = 0
local playerEndTime = 0

function PitBull_VisualHeal:UpdateBars(unit, frame)
	local healthBar = frame.healthBar
	local playerHealBar = frame.playerHealBar
	local incomingHealBar = frame.incomingHealBar
	local incomingHeal
	local isCastingOnThisUnit
	
	if playerIsCasting then
		isCastingOnThisUnit = playerHealingTargetName == frame:GetUnitName()
	end

	if isCastingOnThisUnit then
		incomingHeal = LibHealComm:UnitIncomingHealGet(unit, playerEndTime)
	else
		incomingHeal = LibHealComm:UnitIncomingHealGet(unit, GetTime() + 1000)
	end	
	
	-- Bail out early if nothing going on for this unit
	if not isCastingOnThisUnit and not incomingHeal then
		if playerHealBar then
			playerHealBar.texture = delFrame(playerHealBar.texture)
			playerHealBar.spark = delFrame(playerHealBar.spark)
			frame.playerHealBar = delFrame(playerHealBar)
		end
		if incomingHealBar then
			incomingHealBar.texture = delFrame(incomingHealBar.texture)
			incomingHealBar.spark = delFrame(incomingHealBar.spark)
			frame.incomingHealBar = delFrame(incomingHealBar)
		end
		return
	end

	local healModifier = LibHealComm:UnitHealModifierGet(unit)

	local unitHealthMax = UnitHealthMax(unit)
	if unitHealthMax == 100 then
		-- Estimate
		unitHealthMax = EstimateUnitHealthMax(unit)
	end

	local deficit = not not PitBull_HealthBar.db.profile[frame.group].deficit
	local currentPercent = healthBar:GetValue()
	if deficit then
		currentPercent = 1 - currentPercent
	end
	
	local incomingPercent = incomingHeal and healModifier * incomingHeal / unitHealthMax or 0
	local playerPercent = isCastingOnThisUnit and healModifier * playerHealingSize / unitHealthMax or 0
	
	local healthBar = frame.healthBar
	local frameLevel = healthBar:GetFrameLevel()
	local texture = PitBull:GetStatusBarTexture()
	local height, width = healthBar:GetHeight(), healthBar:GetWidth()
	local orientation = healthBar:GetOrientation()
	local reverse = not not healthBar:GetReverse()
	
	if incomingPercent > 0 then
		if not incomingHealBar then
			incomingHealBar = newFrame("Frame", healthBar)
			frame.incomingHealBar = incomingHealBar
			local incomingHealBar_texture = newFrame("Texture", incomingHealBar, "BACKGROUND")
			incomingHealBar.texture = incomingHealBar_texture
			incomingHealBar_texture:SetTexture(texture)
			incomingHealBar_texture:SetVertexColor(0.4, 0.6, 0.4)
			incomingHealBar_texture:SetAllPoints(incomingHealBar)
			
			local incomingHealBar_spark = newFrame("Texture", incomingHealBar, "OVERLAY")
			incomingHealBar.spark = incomingHealBar_spark
			incomingHealBar_spark:SetTexture([[Interface\CastingBar\UI-CastingBar-Spark]])
			incomingHealBar_spark:SetBlendMode('ADD')
			if orientation ~= "VERTICAL" then
				incomingHealBar_spark:SetPoint('CENTER', incomingHealBar, reverse ~= deficit and "RIGHT" or "LEFT")
				incomingHealBar_spark:SetWidth(16)
				incomingHealBar_spark:SetHeight(height*2)
			else
				incomingHealBar_spark:SetPoint('CENTER', incomingHealBar, reverse ~= deficit and "TOP" or "BOTTOM")
				incomingHealBar_spark:SetHeight(16)
				incomingHealBar_spark:SetWidth(width*2)
				incomingHealBar_spark:SetTexCoord(1, 0, 0, 0, 1, 1, 0, 1)
			end
		end
		incomingHealBar:ClearAllPoints()
		
		if orientation ~= "VERTICAL" then
			incomingHealBar:SetWidth(width * incomingPercent)
			incomingHealBar:SetHeight(height)
			if reverse ~= deficit then
				incomingHealBar.texture:SetTexCoord(incomingPercent, 0, incomingPercent, 1, 0, 0, 0, 1)
			else
				incomingHealBar.texture:SetTexCoord(0, 0, 0, 1, incomingPercent, 0, incomingPercent, 1)
			end
			local n = width * currentPercent
			if reverse ~= deficit then
				incomingHealBar:SetPoint("RIGHT", healthBar, "RIGHT", -n, 0)
			else
				incomingHealBar:SetPoint("LEFT", healthBar, "LEFT", n, 0)
			end
		else
			incomingHealBar:SetWidth(width)
			incomingHealBar:SetHeight(height * incomingPercent)
			if reverse ~= deficit then
				incomingHealBar.texture:SetTexCoord(0, 0, incomingPercent, 0, 0, 1, incomingPercent, 1)
			else
				incomingHealBar.texture:SetTexCoord(incomingPercent, 0, 0, 0, incomingPercent, 1, 0, 1)
			end
			local n = height * currentPercent
			if reverse ~= deficit then
				incomingHealBar:SetPoint("TOP", healthBar, "TOP", 0, -n)
			else
				incomingHealBar:SetPoint("BOTTOM", healthBar, "BOTTOM", 0, n)
			end
		end
	else
		if incomingHealBar then
			incomingHealBar.texture = delFrame(incomingHealBar.texture)
			incomingHealBar.spark = delFrame(incomingHealBar.spark)
			frame.incomingHealBar = delFrame(incomingHealBar)
		end
	end
	
	if playerPercent > 0 then
		if not playerHealBar then
			playerHealBar = newFrame("Frame", healthBar)
			frame.playerHealBar = playerHealBar
			local playerHealBar_texture = newFrame("Texture", playerHealBar, "BACKGROUND")
			playerHealBar.texture = playerHealBar_texture
			playerHealBar_texture:SetTexture(texture)
			playerHealBar_texture:SetVertexColor(0, 1, 0)
			playerHealBar_texture:SetAllPoints(playerHealBar)
			
			playerHealBar:SetWidth(width)
			playerHealBar:SetHeight(height)
			
			local playerHealBar_spark = newFrame("Texture", playerHealBar, "OVERLAY")
			playerHealBar.spark = playerHealBar_spark
			playerHealBar_spark:SetTexture([[Interface\CastingBar\UI-CastingBar-Spark]])
			playerHealBar_spark:SetBlendMode('ADD')
			if orientation ~= "VERTICAL" then
				playerHealBar_spark:SetPoint('CENTER', playerHealBar, reverse ~= deficit and "RIGHT" or "LEFT")
				playerHealBar_spark:SetWidth(16)
				playerHealBar_spark:SetHeight(height*2)
			else
				playerHealBar_spark:SetPoint('CENTER', playerHealBar, reverse ~= deficit and "TOP" or "BOTTOM")
				playerHealBar_spark:SetWidth(width*2)
				playerHealBar_spark:SetHeight(16)
				playerHealBar_spark:SetTexCoord(1, 0, 0, 0, 1, 1, 0, 1)
			end
		end
		local waste
		if currentPercent + incomingPercent > 1 then
			waste = 1
		else
			waste = currentPercent + incomingPercent + playerPercent - 1
			if waste > 0 then
				waste = waste / playerPercent
			else
				waste = 0
			end
		end

		-- Calculate color for overheal severity
		local red = waste > 0.1 and 1 or waste * 10
		local green = waste < 0.1 and 1 or -2.5 * waste + 1.25

		-- Set color for heal
		playerHealBar.texture:SetVertexColor(red, green, 0)

		playerHealBar:ClearAllPoints()
		
		if orientation ~= "VERTICAL" then
			playerHealBar:SetWidth(width * playerPercent)
			playerHealBar:SetHeight(height)
			if reverse ~= deficit then
				playerHealBar.texture:SetTexCoord(playerPercent, 0, playerPercent, 1, 0, 0, 0, 1)
			else
				playerHealBar.texture:SetTexCoord(0, 0, 0, 1, playerPercent, 0, playerPercent, 1)
			end
			
			local n = width * (currentPercent + incomingPercent)
			if reverse ~= deficit then
				playerHealBar:SetPoint("RIGHT", healthBar, "RIGHT", -n, 0)
			else
				playerHealBar:SetPoint("LEFT", healthBar, "LEFT", n, 0)
			end
		else
			playerHealBar:SetWidth(width)
			playerHealBar:SetHeight(height * playerPercent)
			if reverse ~= deficit then
				playerHealBar.texture:SetTexCoord(0, 0, playerPercent, 0, 0, 1, playerPercent, 1)
			else
				playerHealBar.texture:SetTexCoord(playerPercent, 0, 0, 0, playerPercent, 1, 0, 1)
			end
			local n = height * (currentPercent + incomingPercent)
			if reverse ~= deficit then
				playerHealBar:SetPoint("TOP", healthBar, "TOP", 0, -n)
			else
				playerHealBar:SetPoint("BOTTOM", healthBar, "BOTTOM", 0, n)
			end
		end
	else
		if playerHealBar then
			playerHealBar.texture = delFrame(playerHealBar.texture)
			playerHealBar.spark = delFrame(playerHealBar.spark)
			frame.playerHealBar = delFrame(playerHealBar)
		end
	end
end

function PitBull_VisualHeal:UpdateHealth(object, unit, frame)
	self.hooks[object].UpdateHealth(object, unit, frame)
	self:UpdateBars(unit, frame)
end

function PitBull_VisualHeal:OnUpdateLayout(unit, frame)
	self:OnClearUnitFrame(unit, frame)
	self:UpdateBars(unit, frame)
end

function PitBull_VisualHeal:OnClearUnitFrame(unit, frame)
	if frame.playerHealBar then
		frame.playerHealBar.texture = delFrame(frame.playerHealBar.texture)
		frame.playerHealBar.spark = delFrame(frame.playerHealBar.spark)
		frame.playerHealBar = delFrame(frame.playerHealBar)
	end
	if frame.incomingHealBar then
		frame.incomingHealBar.texture = delFrame(frame.incomingHealBar.texture)
		frame.incomingHealBar.spark = delFrame(frame.incomingHealBar.spark)
		frame.incomingHealBar = delFrame(frame.incomingHealBar)
	end
end

function PitBull_VisualHeal:OnUpdateStatusBarTexture(texture)
	for unit,frame in PitBull:IterateUnitFrames() do
		if frame.playerHealBar then
			frame.playerHealBar.texture:SetTexture(texture)
		end
		if frame.incomingHealBar then
			frame.incomingHealBar.texture:SetTexture(texture)
		end
	end
end

function PitBull_VisualHeal:HealComm_DirectHealStart(event, healerName, healSize, endTime, targetName)
	if healerName == playerName then
	 	playerIsCasting = true
		playerHealingTargetName = targetName
		playerHealingSize = healSize
		playerEndTime = endTime
	end
	
	for frame in PitBull:IterateUnitFramesForUnitName(targetName) do
		self:UpdateBars(frame:GetUnit(), frame)
	end	
end

function PitBull_VisualHeal:HealComm_DirectHealUpdate(event, healerName, healSize, endTime, targetName)
	if healerName == playerName then
		playerEndTime = endTime
	end

	for frame in PitBull:IterateUnitFramesForUnitName(targetName) do
		self:UpdateBars(frame:GetUnit(), frame)
	end
end

function PitBull_VisualHeal:HealComm_DirectHealStop(event, healerName, healSize, succeeded, targetName)
	if healerName == playerName then
		playerIsCasting = false
	end

	for frame in PitBull:IterateUnitFramesForUnitName(targetName) do
		self:UpdateBars(frame:GetUnit(), frame)
	end
end

function PitBull_VisualHeal:HealComm_HealModifierUpdate(event, unit, targetName, healModifier)
	for frame in PitBull:IterateUnitFramesForUnit(unit) do				
		self:UpdateBars(unit, frame)
	end
end
